<?xml version='1.0' encoding="windows-1251"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:exsl="http://exslt.org/common" 
                extension-element-prefixes="exsl"
                version='1.0'>


<!-- $Id: collect_images.xsl,v 1.9 2004/08/16 15:32:49 olpa Exp $ -->
<!-- 
          .       .
-->

<xsl:import href="filename.xsl"/>

<!-- Key for finding new image names from old names -->
<!-- Disabled until checking support for exsl:node-set and key by xsltproc
     <xsl:key name="newname" match="image" use="dirfilename"/>
-->

<!-- Params for collect_images using old Makefiles (cp $(shell xsltproc collect_images.xsl ...)) -->
<!-- Disabled by default-->
<xsl:param name='blank-image-if-not-images' select='0'/>
<xsl:param name='blank-image-file'>proc/build-xsl-styles/docbook-xsl/images/blank.png</xsl:param>

<!-- Name for resulting shell script-->
<xsl:param name='collect-images-script'>collect_images.sh</xsl:param>
<!-- Destination for copying images in resulting shell script -->
<xsl:param name='dest-images-dir'>images</xsl:param>
<!-- Prefix for new names of duplicated images -->
<xsl:param name='prefix-rename-images'>image</xsl:param>

<!-- Control xml:base calculation for old libxml2 < 2.6.11
http://lists.altlinux.ru/pipermail/docs/2004-August/004647.html -->
<xsl:param name='old-libxml2' select='0'/>

<xsl:variable name="images-database">
  <xsl:apply-templates select="//graphic|//inlinegraphic|//imagedata" mode="collect-images"/>
</xsl:variable>

<xsl:template name="get-full-base">
  <xsl:param name="node"/>
  <xsl:choose>
    <xsl:when test="$old-libxml2">
      <xsl:value-of select="ancestor-or-self::*[@xml:base][position()=1]/@xml:base"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:for-each select="($node/ancestor-or-self::*[@xml:base])[position()!=last()]" order="descending">
        <xsl:call-template name="dirname">
          <xsl:with-param name="filename" select="@xml:base"/>
        </xsl:call-template>
      </xsl:for-each>
      <xsl:value-of select="($node/ancestor-or-self::*[@xml:base])[position()=last()]/@xml:base"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="graphic|inlinegraphic|imagedata" mode="collect-images">

  <xsl:variable name='current-base'>
    <xsl:call-template name="get-full-base">
      <xsl:with-param name="node" select="."/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name='current-imagename'>
    <xsl:value-of select="@fileref"/>
  </xsl:variable>

  <xsl:variable name='current-dirname'>
    <xsl:call-template name='normalize-path'>
      <xsl:with-param name='path'>
        <xsl:call-template name="dirname">
          <xsl:with-param name="filename" select="$current-base"/>
        </xsl:call-template>
        <xsl:call-template name="dirname">
          <xsl:with-param name="filename" select="$current-imagename"/>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name='current-file'>
    <xsl:call-template name="not-dirname">
      <xsl:with-param name="filename" select="$current-imagename"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="has-preceding-duplicate">
    <xsl:for-each select='preceding::graphic|preceding::inlinegraphic|preceding::imagedata'>
      <xsl:variable name='other-base'>
        <xsl:call-template name="get-full-base">
          <xsl:with-param name="node" select="."/>
        </xsl:call-template>
      </xsl:variable>
      <xsl:variable name='other-imagename'>
        <xsl:value-of select="@fileref"/>
      </xsl:variable>
      <xsl:variable name='other-dirname'>
        <xsl:call-template name='normalize-path'>
          <xsl:with-param name='path'>
            <xsl:call-template name="dirname">
              <xsl:with-param name="filename" select="$other-base"/>
            </xsl:call-template>
            <xsl:call-template name="dirname">
              <xsl:with-param name="filename" select="$other-imagename"/>
            </xsl:call-template>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:variable>
      <xsl:variable name='other-file'>
        <xsl:call-template name="not-dirname">
          <xsl:with-param name="filename" select="$other-imagename"/>
        </xsl:call-template>
      </xsl:variable>
      <!-- FIXME: check xml:base creation for files from same dir in XInclude 2003 -->
      <xsl:if test='$current-file=$other-file and not($current-dirname=$other-dirname)'>
        <xsl:text>1</xsl:text>
      </xsl:if>
    </xsl:for-each>
  </xsl:variable>
  
  <xsl:variable name="has-preceding-self">
    <xsl:for-each select='preceding::graphic|preceding::inlinegraphic|preceding::imagedata'>
      <xsl:variable name='other-base'>
        <xsl:value-of select="ancestor-or-self::*[@xml:base][position()=1]/@xml:base"/>
      </xsl:variable>
      <xsl:variable name='other-imagename'>
        <xsl:value-of select="@fileref"/>
      </xsl:variable>
      <xsl:variable name='other-dirname'>
        <xsl:call-template name='normalize-path'>
          <xsl:with-param name='path'>
            <xsl:call-template name="dirname">
              <xsl:with-param name="filename" select="$other-base"/>
            </xsl:call-template>
            <xsl:call-template name="dirname">
              <xsl:with-param name="filename" select="$other-imagename"/>
            </xsl:call-template>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:variable>
      <xsl:variable name='other-file'>
        <xsl:call-template name="not-dirname">
          <xsl:with-param name="filename" select="$other-imagename"/>
        </xsl:call-template>
      </xsl:variable>
      <!-- FIXME: check xml:base creation for files from same dir in XInclude 2003 -->
      <xsl:if test='$current-file=$other-file and $current-dirname=$other-dirname'>
        <xsl:text>1</xsl:text>
      </xsl:if>
    </xsl:for-each>
  </xsl:variable>

  <xsl:variable name="newimagename">
    <xsl:choose>
      <!-- if has preceding duplicate and not has preceding self, then rename -->
      <xsl:when test="not($has-preceding-duplicate='') and $has-preceding-self=''">
        <xsl:variable name="current-base-output">
          <xsl:choose>
            <xsl:when test="$current-base=''">
              <xsl:text>root document or file from root document dir</xsl:text>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>'</xsl:text>
              <xsl:value-of select="$current-base"/>
              <xsl:text>'</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        <xsl:message>
          <xsl:text>&#10;Warning: image name '</xsl:text>
          <xsl:value-of select="$current-file"/>
          <xsl:text>' from document: </xsl:text>
          <xsl:value-of select="$current-base-output"/>
          <xsl:text> already used and will be renamed in result format.&#10;&#10;</xsl:text>
        </xsl:message>
        <xsl:value-of select="$prefix-rename-images"/>
        <xsl:value-of select="count(preceding::graphic|preceding::inlinegraphic|preceding::imagedata)+1"/>
        <xsl:text>.</xsl:text>
        <xsl:call-template name="file-ext">
          <xsl:with-param name='filename' select="$current-file"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$current-file"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  
  <xsl:if test="$has-preceding-self=''">
    <image>
      <dirfilename><xsl:value-of select="$current-dirname"/><xsl:value-of select="$current-file"/></dirfilename>
      <newname><xsl:value-of select="$newimagename"/></newname>
    </image>
  </xsl:if>

</xsl:template>

<xsl:template match="/">
    <exsl:document href="{$collect-images-script}" method="text"
      encoding="windows-1251">
<xsl:text><![CDATA[#!/bin/sh

SPATH=$1

if [ "$SPATH" = "" ]; then
    SPATH=.
fi

DPATH=$2

if [ "$DPATH" = "" ]; then
    DPATH=.
fi

]]></xsl:text>
<!-- $SPATH/$DPATH used for specifying relative paths, if shell script not in same dir for makefile -->
      <xsl:for-each select="exsl:node-set($images-database)/image">
        <xsl:text>/bin/cp -- "$SPATH/</xsl:text>
        <xsl:value-of select="dirfilename"/>
        <xsl:text>" "$DPATH/</xsl:text>
        <xsl:value-of select="$dest-images-dir"/>
        <xsl:text>/</xsl:text>
        <xsl:value-of select="newname"/>
        <xsl:text>"&#10;</xsl:text>
      </xsl:for-each>
    </exsl:document>
    <xsl:apply-templates/>
</xsl:template>


<xsl:template match="graphic|inlinegraphic|imagedata">

  <xsl:variable name='current-base'>
    <xsl:call-template name="get-full-base">
      <xsl:with-param name="node" select="."/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name='current-imagename'>
    <xsl:value-of select="@fileref"/>
  </xsl:variable>

  <xsl:variable name='current-dirname'>
    <xsl:call-template name='normalize-path'>
      <xsl:with-param name='path'>
        <xsl:call-template name="dirname">
          <xsl:with-param name="filename" select="$current-base"/>
        </xsl:call-template>
        <xsl:call-template name="dirname">
          <xsl:with-param name="filename" select="$current-imagename"/>
        </xsl:call-template>
      </xsl:with-param>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name='current-file'>
    <xsl:call-template name="not-dirname">
      <xsl:with-param name="filename" select="$current-imagename"/>
    </xsl:call-template>
  </xsl:variable>

  <xsl:variable name="current-dirfilename">
    <xsl:value-of select="$current-dirname"/>
    <xsl:value-of select="$current-file"/>
  </xsl:variable>

  <xsl:variable name="new-image-name">
    <!-- FIXME: use key() function ? Test how xsltproc support exsl:node-set and key now -->
    <xsl:value-of select="exsl:node-set($images-database)/image[dirfilename=$current-dirfilename]/newname"/>
  </xsl:variable>

  <xsl:copy>
    <xsl:apply-templates select="@*"/>
    <xsl:attribute name="fileref">
      <xsl:value-of select="$dest-images-dir"/>
      <xsl:text>/</xsl:text>
      <xsl:value-of select="$new-image-name"/>
    </xsl:attribute>
    <xsl:apply-templates select="node()"/>
  </xsl:copy>
</xsl:template>

</xsl:stylesheet>
