<?xml version='1.0' encoding="windows-1251"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:str="http://exslt.org/strings"
                xmlns:exsl="http://exslt.org/common" 
                extension-element-prefixes="str exsl"
                version='1.0'>

<!-- $Id: filename.xsl,v 1.2 2004/08/10 16:54:34 vyt Exp $ -->
<!--     . -->

<xsl:import href="str.split.template.xsl"/>

<xsl:variable name="check-exsl-node-set">
  <xsl:if test="not(function-available('exsl:node-set'))">
    <xsl:message terminate="yes">
      <xsl:text>Error: this style requires support for extension 'exsl:node-set()'</xsl:text>
    </xsl:message>
  </xsl:if>
</xsl:variable>

<!-- Output dirname from full filename path -->
<xsl:template name="dirname">
  <xsl:param name="filename"/>
  <xsl:if test="contains($filename,'/')">
    <xsl:value-of select="substring-before($filename,'/')"/>
    <xsl:text>/</xsl:text>
    <xsl:call-template name="dirname">
      <xsl:with-param name="filename" select="substring-after($filename,'/')"/>
    </xsl:call-template>
  </xsl:if>
</xsl:template>

<!-- Output filename from full filename path -->
<xsl:template name="not-dirname">
  <xsl:param name="filename"/>
  <xsl:choose>
    <xsl:when test="contains($filename,'/')">
      <xsl:call-template name="not-dirname">
        <xsl:with-param name="filename" select="substring-after($filename,'/')"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$filename"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- Output filename extension from full filename path -->
<xsl:template name="file-ext">
  <xsl:param name="filename"/>
  <xsl:choose>
    <xsl:when test="contains($filename,'.')">
      <xsl:call-template name="file-ext">
        <xsl:with-param name="filename" select="substring-after($filename,'.')"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$filename"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- Template for normalize path from updirs (../some/path/../dir/images) -->
<xsl:template name='normalize-path'>
  <xsl:param name='path'/>
  <xsl:choose>
    <xsl:when test="contains($path,'../')">
      <xsl:variable name='tokens'>
        <xsl:call-template name='str:split'>
          <xsl:with-param name='string' select='$path'/>
          <xsl:with-param name='pattern' select="'/'"/>
        </xsl:call-template>
      </xsl:variable>
      <xsl:call-template name='strip-updir-beforedir'>
        <xsl:with-param name='tokens' select='$tokens'/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select='$path'/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name='strip-updir-beforedir'>
  <xsl:param name='tokens'/>
  <xsl:choose>
    <xsl:when test="exsl:node-set($tokens)/token[.='..'][preceding-sibling::token[1][not(.='..')]]">
      <xsl:variable name='updir-position'>
        <xsl:value-of select="count((exsl:node-set($tokens)/token[.='..'][preceding-sibling::token[1][not(.='..')]])[1]/preceding-sibling::token)+1"/>
      </xsl:variable>
      <xsl:call-template name='strip-updir-beforedir'>
        <xsl:with-param name='tokens'>
          <xsl:for-each select="exsl:node-set($tokens)/token">
            <xsl:if test="position() != $updir-position and position() != ($updir-position - 1)">
              <token><xsl:value-of select='.'/></token>
            </xsl:if>
          </xsl:for-each>
        </xsl:with-param>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:for-each select="exsl:node-set($tokens)/token">
        <xsl:if test="not(.='')">
          <xsl:value-of select='.'/>
          <xsl:if test="not(position()=last())">
            <xsl:text>/</xsl:text>
          </xsl:if>
        </xsl:if>
      </xsl:for-each>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

</xsl:stylesheet>
